@@ -92,19 +92,13 @@ glade_gnome_app_post_create (GladeWidgetAdaptor *adaptor,
 			     GladeCreateReason   reason)
 {
 	static GladeWidgetAdaptor *menubar_adaptor = NULL, *dock_item_adaptor;
-	GladeWidget *gapp, *gdock, *gdock_item, *gmenubar;
-	GladeProject *project;
-	GnomeApp *app;
-	
-	g_return_if_fail (GNOME_IS_APP (object));
+	GnomeApp *app = GNOME_APP (object);
+	GladeWidget *gapp = glade_widget_get_from_gobject (object);
+	GladeProject *project = glade_widget_get_project (gapp);
+	GladeWidget *gdock, *gdock_item, *gmenubar;
 	
 	gtk_window_set_default_size (GTK_WINDOW (object), 440, 250);
 	
-	app = GNOME_APP (object);
-	gapp = glade_widget_get_from_gobject (object);
-	
-	project = glade_widget_get_project (gapp);
-	
 	/* Add BonoboDock */
 	gdock = glade_widget_adaptor_create_internal
 		(gapp, G_OBJECT (app->dock),
@@ -152,13 +146,8 @@ glade_gnome_app_get_internal_child (GladeWidgetAdaptor  *adaptor,
 				    const gchar         *name)
 {
 	GObject     *child = NULL;
-	GnomeApp    *app;
-	GladeWidget *gapp;
-	
-	g_return_val_if_fail (GNOME_IS_APP (object), NULL);
-	
-	app = GNOME_APP (object);
-	gapp = glade_widget_get_from_gobject (object);
+	GnomeApp    *app   = GNOME_APP (object);
+	GladeWidget *gapp  = glade_widget_get_from_gobject (object);
 	
 	if (strcmp ("dock", name) == 0)
 		child = G_OBJECT (app->dock);
@@ -167,7 +156,12 @@ glade_gnome_app_get_internal_child (GladeWidgetAdaptor  *adaptor,
 		child = G_OBJECT (app->statusbar);
 		if (child == NULL)
 		{
-			/* XXX Is this appropriate ? */
+			/*
+			  Create appbar, libglade handle this as an internal
+			  widget but appbar is not created by GnomeApp.
+			  So we need to create it here for loading purpouses since
+			  "has-statusbar" property is not saved in the .glade file.
+			*/
 			glade_widget_property_set (gapp, "has-statusbar", TRUE);
 			child = G_OBJECT (app->statusbar);
 		}
@@ -180,12 +174,8 @@ GList * GLADEGNOME_API
 glade_gnome_app_get_children (GladeWidgetAdaptor  *adaptor,
 			      GObject             *object)
 {
+	GnomeApp *app = GNOME_APP (object);
 	GList *list = NULL;
-	GnomeApp *app;
-
-	g_return_val_if_fail (GNOME_IS_APP (object), NULL);
-	
-	app = GNOME_APP (object);
 	
 	if (app->dock) list = g_list_append (list, G_OBJECT (app->dock));
 	if (app->statusbar) list = g_list_append (list, G_OBJECT (app->statusbar));
@@ -201,13 +191,10 @@ glade_gnome_app_set_child_property (GladeWidgetAdaptor  *adaptor,
 				    const gchar         *property_name,
 				    GValue              *value)
 {
-	GnomeApp *app;
+	GnomeApp *app = GNOME_APP (container);
 	
-	g_return_if_fail (GNOME_IS_APP (container));
 	g_return_if_fail (GTK_IS_WIDGET (child));
 	
-	app = GNOME_APP (container);
-
 	gtk_container_child_set_property (GTK_CONTAINER (app->vbox), 
 					  (GNOME_IS_APPBAR (child)) 
 						? gtk_widget_get_parent (GTK_WIDGET (child))
@@ -223,13 +210,10 @@ glade_gnome_app_get_child_property (GladeWidgetAdaptor  *adaptor,
 				    const gchar         *property_name,
 				    GValue              *value)
 {
-	GnomeApp *app;
+	GnomeApp *app = GNOME_APP (container);
 	
-	g_return_if_fail (GNOME_IS_APP (container));
 	g_return_if_fail (GTK_IS_WIDGET (child));
 	
-	app = GNOME_APP (container);
-
 	gtk_container_child_get_property (GTK_CONTAINER (app->vbox), 
 					  (GNOME_IS_APPBAR (child)) 
 						? gtk_widget_get_parent (GTK_WIDGET (child))
@@ -238,16 +222,11 @@ glade_gnome_app_get_child_property (GladeWidgetAdaptor  *adaptor,
 					  value);
 }
 
-void GLADEGNOME_API
-glade_gnome_app_set_has_statusbar (GObject *object, GValue *value)
+static void
+glade_gnome_app_set_has_statusbar (GObject *object, const GValue *value)
 {
-	GnomeApp *app;
-	GladeWidget *gapp, *gbar;
-	
-	g_return_if_fail (GNOME_IS_APP (object));
-	
-	app = GNOME_APP (object);
-	gapp = glade_widget_get_from_gobject (object);
+	GnomeApp *app = GNOME_APP (object);
+	GladeWidget *gapp = glade_widget_get_from_gobject (object), *gbar;
 	
 	if (g_value_get_boolean (value))
 	{
@@ -271,6 +250,7 @@ glade_gnome_app_set_has_statusbar (GObject *object, GValue *value)
 	{
 		if (app->statusbar)
 		{
+			/* FIXME: we need to destroy GladeWidgets too */
 			glade_project_remove_object (glade_widget_get_project (gapp),
 						     G_OBJECT (app->statusbar));
 			gtk_container_remove (GTK_CONTAINER (app->vbox),
@@ -280,6 +260,23 @@ glade_gnome_app_set_has_statusbar (GObject *object, GValue *value)
 	}
 }
 
+
+void GLADEGNOME_API
+glade_gnome_app_set_property (GladeWidgetAdaptor *adaptor,
+			      GObject            *object,
+			      const gchar        *id,
+			      const GValue       *value)
+{
+	if (!strcmp (id, "has-statusbar"))
+		glade_gnome_app_set_has_statusbar (object, value);
+	else if (!strcmp (id, "enable-layout-config"))
+		/* do nothing */;
+	else
+		GWA_GET_CLASS (GTK_TYPE_WINDOW)->set_property (adaptor,
+							       object,
+							       id, value);
+}
+
 /* GnomeAppBar */
 GType GLADEGNOME_API
 gnome_app_bar_get_type ()
@@ -292,7 +289,6 @@ glade_gnome_app_bar_post_create (GladeWidgetAdaptor  *adaptor,
 				 GObject             *object, 
 				 GladeCreateReason    reason)
 {
-	g_return_if_fail (GNOME_IS_APPBAR (object));
 	gnome_appbar_set_status (GNOME_APPBAR (object), _("Status Message."));
 }
 
@@ -308,8 +304,6 @@ glade_gnome_date_edit_post_create (GladeWidgetAdaptor  *adaptor,
 				   GObject             *object, 
 				   GladeCreateReason    reason)
 {
-	g_return_if_fail (GNOME_IS_DATE_EDIT (object));
-
 	/* DateEdit's "dateedit-flags" property hides/shows some widgets so we 
 	 * need to explicitly tell that they should not be affected by 
 	 * gtk_widget_show_all() (its, for example, called after a paste)
@@ -348,8 +342,6 @@ glade_gnome_druid_post_create (GladeWidgetAdaptor  *adaptor,
 {
 	GladeWidget *gdruid, *gpage;
 
-	g_return_if_fail (GNOME_IS_DRUID (object));
-	
 	if (reason != GLADE_CREATE_USER) return;
 
 	gdruid = glade_widget_get_from_gobject (object);
@@ -411,10 +403,12 @@ glade_gnome_druid_add_child (GladeWidgetAdaptor  *adaptor,
 			     GObject             *container, 
 			     GObject             *child)
 {
-	g_return_if_fail (GNOME_IS_DRUID (container));
 	g_return_if_fail (GNOME_IS_DRUID_PAGE (child));
 
-	/* XXX This needed ? */
+	/*
+	  Disconnect handlers just in case this child was already added
+	  in another druid.
+	 */
 	g_signal_handlers_disconnect_matched (child, G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
 					      glade_gnome_druid_page_cb, NULL);
 
@@ -431,7 +425,6 @@ glade_gnome_druid_remove_child (GladeWidgetAdaptor  *adaptor,
 				GObject             *container, 
 				GObject             *child)
 {
-	g_return_if_fail (GNOME_IS_DRUID (container));
 	g_return_if_fail (GNOME_IS_DRUID_PAGE (child));
 
 	g_signal_handlers_disconnect_matched (child, G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
@@ -491,7 +484,6 @@ glade_gnome_druid_set_child_property (GladeWidgetAdaptor  *adaptor,
 				      const gchar         *property_name,
 				      GValue              *value)
 {
-	g_return_if_fail (GNOME_IS_DRUID (container));
 	g_return_if_fail (GNOME_IS_DRUID_PAGE (child));
 	
 	if (strcmp (property_name, "position") == 0)
@@ -517,8 +509,8 @@ glade_gnome_druid_set_child_property (GladeWidgetAdaptor  *adaptor,
 		/* Chain Up */
 		GWA_GET_CLASS
 			(GTK_TYPE_CONTAINER)->child_set_property (adaptor,
-								  GTK_CONTAINER (container), 
-								  GTK_WIDGET (child),
+								  container, 
+								  child,
 								  property_name,
 								  value);
 }
@@ -530,7 +522,6 @@ glade_gnome_druid_get_child_property (GladeWidgetAdaptor  *adaptor,
 				      const gchar         *property_name,
 				      GValue              *value)
 {
-	g_return_if_fail (GNOME_IS_DRUID (container));
 	g_return_if_fail (GNOME_IS_DRUID_PAGE (child));
 	
 	if (strcmp (property_name, "position") == 0)
@@ -541,8 +532,8 @@ glade_gnome_druid_get_child_property (GladeWidgetAdaptor  *adaptor,
 		/* Chain Up */
 		GWA_GET_CLASS
 			(GTK_TYPE_CONTAINER)->child_get_property (adaptor,
-								  GTK_CONTAINER (container), 
-								  GTK_WIDGET (child),
+								  container, 
+								  child,
 								  property_name,
 								  value);
 }
@@ -556,8 +547,6 @@ glade_gnome_dps_post_create (GladeWidgetAdaptor  *adaptor,
 	GladeWidget *gpage, *gvbox;
 	GObject *vbox;
 	
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_STANDARD (object));
-	
 	gpage = glade_widget_get_from_gobject (object);
 	vbox = G_OBJECT (GNOME_DRUID_PAGE_STANDARD (object)->vbox);
 	gvbox = glade_widget_adaptor_create_internal (gpage, vbox, "vbox",
@@ -574,8 +563,6 @@ glade_gnome_dps_get_internal_child (GladeWidgetAdaptor  *adaptor,
 				    const gchar         *name)
 {
 	GObject *child = NULL;
-
-	g_return_val_if_fail (GNOME_IS_DRUID_PAGE_STANDARD (object), NULL);
 	
 	if (strcmp (name, "vbox") == 0)
 		child = G_OBJECT (GNOME_DRUID_PAGE_STANDARD (object)->vbox);
@@ -587,29 +574,23 @@ GList * GLADEGNOME_API
 glade_gnome_dps_get_children (GladeWidgetAdaptor  *adaptor,
 			      GObject             *object)
 {
+	GnomeDruidPageStandard *page = GNOME_DRUID_PAGE_STANDARD (object);
 	GList *list = NULL;
-	GnomeDruidPageStandard *page;
 	
-	g_return_val_if_fail (GNOME_IS_DRUID_PAGE_STANDARD (object), NULL);
-	
-	page = GNOME_DRUID_PAGE_STANDARD (object);
-
 	if (page->vbox) list = g_list_append (list, G_OBJECT (page->vbox));
 		
 	return list;
 }
 
 static void
-glade_gnome_dps_set_color_common (GObject *object,
-				  const gchar *property_name,
-				  GValue *value)
+glade_gnome_dps_set_color_common (GObject      *object,
+				  const gchar  *property_name,
+				  const GValue *value)
 {
 	GladeProperty *prop;
 	const gchar *color_str;
 	GValue *color;
 	
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_STANDARD (object));
-	
 	if ((color_str = g_value_get_string (value)) == NULL) return;
 
 	prop = glade_widget_get_property (glade_widget_get_from_gobject (object),
@@ -621,28 +602,29 @@ glade_gnome_dps_set_color_common (GObject *object,
 }
 
 void GLADEGNOME_API
-glade_gnome_dps_set_background (GObject *object, GValue *value)
+glade_gnome_dps_set_property (GladeWidgetAdaptor *adaptor,
+			      GObject            *object,
+			      const gchar        *id,
+			      const GValue       *value)
 {
-	glade_gnome_dps_set_color_common (object, "background-gdk", value);
+
+	if (!strcmp (id, "background"))
+		glade_gnome_dps_set_color_common (object, "background-gdk", value);
+	else if (!strcmp (id, "contents-background"))
+		glade_gnome_dps_set_color_common (object, "contents-background-gdk", value);
+	else if (!strcmp (id, "logo-background"))
+		glade_gnome_dps_set_color_common (object, "logo-background-gdk", value);
+	else if (!strcmp (id, "title-foreground"))
+		glade_gnome_dps_set_color_common (object, "title-foreground-gdk", value);
+	else
+		/* Skip GNOME_TYPE_DRUID_PAGE since we didnt register an
+		 * adaptor for that abstract class.
+		 */
+		GWA_GET_CLASS (GTK_TYPE_CONTAINER)->set_property (adaptor,
+								  object,
+								  id, value);
 }
 
-void GLADEGNOME_API
-glade_gnome_dps_set_contents_background (GObject *object, GValue *value)
-{
-	glade_gnome_dps_set_color_common (object, "contents-background-gdk", value);
-}
-
-void GLADEGNOME_API
-glade_gnome_dps_set_logo_background (GObject *object, GValue *value)
-{
-	glade_gnome_dps_set_color_common (object, "logo-background-gdk", value);
-}
-
-void GLADEGNOME_API
-glade_gnome_dps_set_title_foreground (GObject *object, GValue *value)
-{
-	glade_gnome_dps_set_color_common (object, "title-foreground-gdk", value);
-}
 
 /* GnomeDruidPageEdge */
 static GType
@@ -671,129 +653,82 @@ glade_gnome_dpe_position_spec (void)
 }
 
 void GLADEGNOME_API
-glade_gnome_dpe_set_title (GObject *object, GValue *value)
+glade_gnome_dpe_set_property (GladeWidgetAdaptor *adaptor,
+			      GObject            *object,
+			      const gchar        *id,
+			      const GValue       *value)
+
 {
-	const gchar *title;
+	GnomeDruidPageEdge *page = GNOME_DRUID_PAGE_EDGE (object);
+	const gchar *text = NULL;
+	GObject *pixbuf = NULL;
+	GdkColor *color = NULL;
 	
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
+	if (G_VALUE_HOLDS (value, G_TYPE_STRING))
+		text = g_value_get_string (value);
+	else if (G_VALUE_HOLDS (value, GDK_TYPE_PIXBUF))
+		pixbuf = g_value_get_object (value);
+	else if (G_VALUE_HOLDS (value, GDK_TYPE_COLOR))
+		color = (GdkColor *) g_value_get_boxed (value);
 	
-	if ((title = g_value_get_string (value)))
-		gnome_druid_page_edge_set_title (GNOME_DRUID_PAGE_EDGE (object),
-						 title);
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_position (GObject *object, GValue *value)
-{
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_text (GObject *object, GValue *value)
-{
-	const gchar *text;
-	
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-	
-	if ((text = g_value_get_string (value)))
-		gnome_druid_page_edge_set_text (GNOME_DRUID_PAGE_EDGE (object),
-						text);
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_title_foreground (GObject *object, GValue *value)
-{
-	GdkColor *color;
-
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-
-	if ((color = (GdkColor *) g_value_get_boxed (value)))
-		gnome_druid_page_edge_set_title_color (GNOME_DRUID_PAGE_EDGE (object),
-						       color);
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_text_foreground (GObject *object, GValue *value)
-{
-	GdkColor *color;
-
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-
-	if ((color = (GdkColor *) g_value_get_boxed (value)))
-		gnome_druid_page_edge_set_text_color (GNOME_DRUID_PAGE_EDGE (object),
-						      color);
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_background (GObject *object, GValue *value)
-{
-	GdkColor *color;
-
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-
-	if ((color = (GdkColor *) g_value_get_boxed (value)))
-		gnome_druid_page_edge_set_bg_color (GNOME_DRUID_PAGE_EDGE (object),
-						    color);
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_contents_background (GObject *object, GValue *value)
-{
-	GdkColor *color;
-
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-
-	if ((color = (GdkColor *) g_value_get_boxed (value)))
-		gnome_druid_page_edge_set_textbox_color (GNOME_DRUID_PAGE_EDGE (object),
-							 color);
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_logo_background (GObject *object, GValue *value)
-{
-	GdkColor *color;
-
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-
-	if ((color = (GdkColor *) g_value_get_boxed (value)))
-		gnome_druid_page_edge_set_logo_bg_color (GNOME_DRUID_PAGE_EDGE (object),
-							 color);
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_logo (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-	
-	gnome_druid_page_edge_set_logo (GNOME_DRUID_PAGE_EDGE (object),
-					GDK_PIXBUF (g_value_get_object (value)));
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_watermark (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-	
-	gnome_druid_page_edge_set_logo (GNOME_DRUID_PAGE_EDGE (object),
-					GDK_PIXBUF (g_value_get_object (value)));
-}
-
-void GLADEGNOME_API
-glade_gnome_dpe_set_top_watermark (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_DRUID_PAGE_EDGE (object));
-	
-	gnome_druid_page_edge_set_top_watermark (GNOME_DRUID_PAGE_EDGE (object),
-						 GDK_PIXBUF (g_value_get_object (value)));
+	if (!strcmp (id, "title"))
+	{
+		if (text) gnome_druid_page_edge_set_title (page, text);
+	}
+	else if (!strcmp (id, "text"))
+	{
+		if (text) gnome_druid_page_edge_set_text (page, text);
+	}
+	else if (!strcmp (id, "title-foreground"))
+	{
+		if (color) gnome_druid_page_edge_set_title_color (page, color);
+	}
+	else if (!strcmp (id, "text-foreground"))
+	{
+		if (color) gnome_druid_page_edge_set_text_color (page, color);
+	}
+	else if (!strcmp (id, "background"))
+	{
+		if (color) gnome_druid_page_edge_set_bg_color (page, color);
+	}
+	else if (!strcmp (id, "contents-background"))
+	{
+		if (color) gnome_druid_page_edge_set_textbox_color (page, color);
+	}
+	else if (!strcmp (id, "logo-background"))
+	{
+		if (color) gnome_druid_page_edge_set_logo_bg_color (page, color);
+	}
+	else if (!strcmp (id, "logo"))
+		gnome_druid_page_edge_set_logo (page, GDK_PIXBUF (pixbuf));
+	else if (!strcmp (id, "watermark"))
+		gnome_druid_page_edge_set_watermark (page, GDK_PIXBUF (pixbuf));
+	else if (!strcmp (id, "top-watermark"))
+		gnome_druid_page_edge_set_top_watermark (page, GDK_PIXBUF (pixbuf));
+	else if (!strcmp (id, "position"));
+	else
+		/* Skip GNOME_TYPE_DRUID_PAGE since we didnt register an
+		 * adaptor for that abstract class.
+		 */
+		GWA_GET_CLASS (GTK_TYPE_CONTAINER)->set_property (adaptor,
+								  object,
+								  id, value);
 }
 
 /* GnomeIconEntry */
 void GLADEGNOME_API
-glade_gnome_icon_entry_set_max (GObject *object, GValue *value)
+glade_gnome_icon_entry_set_property (GladeWidgetAdaptor *adaptor,
+				     GObject            *object,
+				     const gchar        *id,
+				     const GValue       *value)
 {
-	g_return_if_fail (GNOME_IS_ICON_ENTRY (object));
-	
-	gnome_icon_entry_set_max_saved (GNOME_ICON_ENTRY (object),
-					g_value_get_uint (value));
+	if (!strcmp (id, "max-saved"))
+		gnome_icon_entry_set_max_saved (GNOME_ICON_ENTRY (object),
+						g_value_get_uint (value));
+	else
+		GWA_GET_CLASS (GTK_TYPE_VBOX)->set_property (adaptor,
+							     object,
+							     id, value);
 }
 
 /* GnomeCanvas */
@@ -806,14 +741,12 @@ typedef enum
 }GnomeCanvasCoordinate;
 
 static void
-glade_gnome_canvas_set_coordinate_common (GObject *object,
-					  GValue *value,
-					  GnomeCanvasCoordinate coordinate)
+glade_gnome_canvas_set_coordinate_common (GObject               *object,
+					  const GValue          *value,
+					  GnomeCanvasCoordinate  coordinate)
 {
 	gdouble x1, y1, x2, y2;
 	
-	g_return_if_fail (GNOME_IS_CANVAS (object));
-	
 	gnome_canvas_get_scroll_region  (GNOME_CANVAS (object),
 					 &x1, &y1, &x2, &y2);
 
@@ -837,36 +770,26 @@ glade_gnome_canvas_set_coordinate_common (GObject *object,
 }
 
 void GLADEGNOME_API
-glade_gnome_canvas_set_coordinate_x1 (GObject *object, GValue *value)
+glade_gnome_canvas_set_property (GladeWidgetAdaptor *adaptor,
+				 GObject            *object,
+				 const gchar        *id,
+				 const GValue       *value)
 {
-	glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_X1);
-}
-
-void GLADEGNOME_API
-glade_gnome_canvas_set_coordinate_y1 (GObject *object, GValue *value)
-{
-	glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_Y1);
-}
-
-void GLADEGNOME_API
-glade_gnome_canvas_set_coordinate_x2 (GObject *object, GValue *value)
-{
-	glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_X2);
-}
-
-void GLADEGNOME_API
-glade_gnome_canvas_set_coordinate_y2 (GObject *object, GValue *value)
-{
-	glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_Y2);
-}
-
-void GLADEGNOME_API
-glade_gnome_canvas_set_pixels (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_CANVAS (object));
-	
-	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (object),
-					  g_value_get_float (value));
+	if (!strcmp (id, "pixels-per-unit"))
+		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (object),
+						  g_value_get_float (value));
+	else if (!strcmp (id, "scroll-x1"))
+		glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_X1);
+	else if (!strcmp (id, "scroll-x2"))
+		glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_X2);
+	else if (!strcmp (id, "scroll-y1"))
+		glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_Y1);
+	else if (!strcmp (id, "scroll-y2"))
+		glade_gnome_canvas_set_coordinate_common (object, value, CANVAS_Y2);
+	else
+		GWA_GET_CLASS (GTK_TYPE_LAYOUT)->set_property (adaptor,
+							       object,
+							       id, value);
 }
 
 /* GnomeDialog */
@@ -908,14 +831,10 @@ glade_gnome_dialog_post_create (GladeWidgetAdaptor  *adaptor,
 				GObject             *object, 
 				GladeCreateReason    reason)
 {
-	GladeWidget *gdialog, *gvbox, *gaction_area;
+	GladeWidget *gdialog = glade_widget_get_from_gobject (object);
+	GnomeDialog *dialog = GNOME_DIALOG (object);
+	GladeWidget *gvbox, *gaction_area;
 	GtkWidget *separator;
-	GnomeDialog *dialog;
-	
-	g_return_if_fail (GNOME_IS_DIALOG (object));
-	
-	dialog = GNOME_DIALOG (object);
-	gdialog = glade_widget_get_from_gobject (object);
 	
 	/* Ignore close signal */
 	g_signal_connect (object, "close", G_CALLBACK (gtk_true), NULL);
@@ -995,9 +914,7 @@ glade_gnome_dialog_get_internal_child (GladeWidgetAdaptor  *adaptor,
 				       const gchar         *name)
 {
 	GObject *child = NULL;
-
-	g_return_val_if_fail (GNOME_IS_DIALOG (object), NULL);
-
+	
 	if (strcmp (name, "vbox") == 0)
 		child = G_OBJECT (GNOME_DIALOG (object)->vbox);
 	else if (GNOME_IS_PROPERTY_BOX (object) && strcmp (name, "notebook") == 0)
@@ -1010,12 +927,8 @@ GList * GLADEGNOME_API
 glade_gnome_dialog_get_children (GladeWidgetAdaptor  *adaptor,
 				 GObject             *object)
 {
+	GnomeDialog *dialog = GNOME_DIALOG (object);
 	GList *list = NULL;
-	GnomeDialog *dialog;
-	
-	g_return_val_if_fail (GNOME_IS_DIALOG (object), NULL);
-	
-	dialog = GNOME_DIALOG (object);
 
 	if (dialog->vbox) list = g_list_append (list, G_OBJECT (dialog->vbox));
 	
@@ -1035,45 +948,25 @@ glade_gnome_about_dialog_post_create (GladeWidgetAdaptor  *adaptor,
 				      GObject             *object, 
 				      GladeCreateReason    reason)
 {
-	g_return_if_fail (GNOME_IS_ABOUT (object));
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (object), GTK_RESPONSE_CLOSE, FALSE);	
 }
 
-GList * GLADEGNOME_API
-glade_gnome_about_dialog_get_children (GladeWidgetAdaptor  *adaptor,
-				       GObject             *object)
-{
-	/* XXX Why this ? */
-	return NULL;
-}
-
 void GLADEGNOME_API
-glade_gnome_about_set_name (GObject *object, GValue *value)
+glade_gnome_about_dialog_set_property (GladeWidgetAdaptor *adaptor,
+				       GObject            *object,
+				       const gchar        *id,
+				       const GValue       *value)
 {
-	g_return_if_fail (GNOME_IS_ABOUT (object));
-	
-	if (g_value_get_string (value))
-		g_object_set_property (object, "name", value);
-}
-
-void GLADEGNOME_API
-glade_gnome_about_set_version (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_ABOUT (object));
-	
-	if (g_value_get_string (value))
-		g_object_set_property (object, "version", value);
-}
-
-/* GnomeIconList */
-void GLADEGNOME_API
-glade_gnome_icon_list_post_create (GladeWidgetAdaptor  *adaptor,
-				   GObject             *object, 
-				   GladeCreateReason    reason)
-{
-	g_return_if_fail (GNOME_IS_ICON_LIST (object));
-	/* Freeze the widget so we dont get the signals that cause a segfault */
-	gnome_icon_list_freeze (GNOME_ICON_LIST (object));
+	if (!strcmp (id, "name") ||
+	    !strcmp (id, "version"))
+	{
+		if (g_value_get_string (value))
+			g_object_set_property (object, id, value);
+	}
+	else
+		GWA_GET_CLASS (GTK_TYPE_DIALOG)->set_property (adaptor,
+							       object,
+							       id, value);
 }
 
 /* GnomeMessageBox */
@@ -1137,9 +1030,9 @@ glade_gnome_message_clean (GObject *object)
 	GtkContainer *container = GTK_CONTAINER (GNOME_DIALOG (object)->vbox);
 	GList *children, *l;
 
-	children = l = gtk_container_get_children (container);
+	children = gtk_container_get_children (container);
 	
-	while (l)
+	for (l = children; l; l = l->next)
 	{
 		GtkWidget *child = (GtkWidget *) l->data;
 		
@@ -1148,20 +1041,16 @@ glade_gnome_message_clean (GObject *object)
 			gtk_container_remove (container, child);
 			break;
 		}
-		
-		l = l->next;
 	}
 	
 	g_list_free (children);
 }
 	
-void GLADEGNOME_API
-glade_gnome_message_box_set_type (GObject *object, GValue *value)
+static void
+glade_gnome_message_box_set_type (GObject *object, const GValue *value)
 {
 	gchar *message, *type;
 	
-	g_return_if_fail (GNOME_IS_MESSAGE_BOX (object));
-	
 	glade_gnome_message_clean (object);
 	glade_widget_property_get (glade_widget_get_from_gobject (object),
 				   "message", &message);
@@ -1170,12 +1059,10 @@ glade_gnome_message_box_set_type (GObject *object, GValue *value)
 				     message, type, NULL); 
 }
 
-void GLADEGNOME_API
-glade_gnome_message_box_set_message (GObject *object, GValue *value)
+static void
+glade_gnome_message_box_set_message (GObject *object, const GValue *value)
 {
 	GladeGnomeMessageBoxType type;
-
-	g_return_if_fail (GNOME_IS_MESSAGE_BOX (object));
 	
 	glade_gnome_message_clean (object);
 	glade_widget_property_get (glade_widget_get_from_gobject (object),
@@ -1186,6 +1073,22 @@ glade_gnome_message_box_set_message (GObject *object, GValue *value)
 				     NULL);	
 }
 
+void GLADEGNOME_API
+glade_gnome_message_box_set_property (GladeWidgetAdaptor *adaptor,
+				      GObject            *object,
+				      const gchar        *id,
+				      const GValue       *value)
+{
+	if (!strcmp (id, "message-box-type"))
+		glade_gnome_message_box_set_type (object, value);
+	else if (!strcmp (id, "message"))
+		glade_gnome_message_box_set_message (object, value);
+	else
+		GWA_GET_CLASS (GNOME_TYPE_DIALOG)->set_property (adaptor,
+								 object,
+								 id, value);
+}
+
 /* GnomeEntry & GnomeFileEntry */
 /* GnomeFileEntry is not derived from GnomeEntry... but hey!!! they should :) */
 GObject * GLADEGNOME_API
@@ -1195,9 +1098,6 @@ glade_gnome_entry_get_internal_child (GladeWidgetAdaptor  *adaptor,
 {
 	GObject *child = NULL;
 
-	g_return_val_if_fail (GNOME_IS_ENTRY (object) || 
-			      GNOME_IS_FILE_ENTRY (object), NULL);
-
 	if (strcmp (name, "entry") == 0)
 	{
 		if (GNOME_IS_ENTRY (object))
@@ -1217,8 +1117,6 @@ glade_gnome_entry_post_create (GladeWidgetAdaptor  *adaptor,
 	GladeWidget *gentry;
 	GObject *child;
 
-	g_return_if_fail (GNOME_IS_ENTRY (object) || GNOME_IS_FILE_ENTRY (object));
-
 	child = glade_gnome_entry_get_internal_child (adaptor, object, "entry");
 	
 	gentry = glade_widget_get_from_gobject (object);
@@ -1235,8 +1133,6 @@ glade_gnome_entry_get_children (GladeWidgetAdaptor  *adaptor,
 	GList *list = NULL;
 	GtkWidget *entry;
 	
-	g_return_val_if_fail (GNOME_IS_ENTRY (object) || GNOME_IS_FILE_ENTRY (object), NULL);
-
 	if (GNOME_IS_ENTRY (object))
 		entry = gnome_entry_gtk_entry (GNOME_ENTRY (object));
 	else
@@ -1248,37 +1144,55 @@ glade_gnome_entry_get_children (GladeWidgetAdaptor  *adaptor,
 }
 
 void GLADEGNOME_API
-glade_gnome_entry_set_max_saved (GObject *object, GValue *value)
+glade_gnome_entry_set_property (GladeWidgetAdaptor *adaptor,
+				GObject            *object,
+				const gchar        *id,
+				const GValue       *value)
 {
-	g_return_if_fail (GNOME_IS_ENTRY (object) || GNOME_IS_FILE_ENTRY (object));
-	
-	if (GNOME_IS_FILE_ENTRY (object))
-		object = G_OBJECT (gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (object)));
+	if (!strcmp (id, "max-saved"))
+		gnome_entry_set_max_saved (GNOME_ENTRY (object), g_value_get_int (value));
+	else
+		GWA_GET_CLASS (GTK_TYPE_COMBO)->set_property (adaptor,
+							      object,
+							      id, value);
+}
 
-	gnome_entry_set_max_saved (GNOME_ENTRY (object), g_value_get_int (value));
+void GLADEGNOME_API
+glade_gnome_file_entry_set_property (GladeWidgetAdaptor *adaptor,
+				     GObject            *object,
+				     const gchar        *id,
+				     const GValue       *value)
+{
+	GtkWidget *entry;
+
+	if (!strcmp (id, "max-saved"))
+	{
+		entry = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (object));
+		gnome_entry_set_max_saved (GNOME_ENTRY (entry), g_value_get_int (value));
+	}
+	else
+		GWA_GET_CLASS (GTK_TYPE_VBOX)->set_property (adaptor,
+							     object,
+							     id, value);
 }
 
 /* GnomePixmapEntry */
 void GLADEGNOME_API
 glade_gnome_pixmap_entry_set_do_preview (GObject *object, GValue *value)
 {
-	g_return_if_fail (GNOME_IS_PIXMAP_ENTRY (object));
-	
 	gnome_pixmap_entry_set_preview (GNOME_PIXMAP_ENTRY (object),
 					g_value_get_boolean (value));
 }
 
 /* GnomeFontPicker */
-void GLADEGNOME_API
-glade_gnome_font_picker_set_mode (GObject *object, GValue *value)
+static void
+glade_gnome_font_picker_set_mode (GObject *object, const GValue *value)
 {
 	GladeWidget *ggfp, *gchild;
 	GnomeFontPicker *gfp;
 	GnomeFontPickerMode mode;
 	GObject *child;
 	
-	g_return_if_fail (GNOME_IS_FONT_PICKER (object));
-	
 	mode = g_value_get_enum (value);
 	if (mode == GNOME_FONT_PICKER_MODE_UNKNOWN) return;
 	
@@ -1310,14 +1224,26 @@ glade_gnome_font_picker_set_mode (GObject *object, GValue *value)
 	}
 }
 
+void GLADEGNOME_API
+glade_gnome_font_picker_set_property (GladeWidgetAdaptor *adaptor,
+				      GObject            *object,
+				      const gchar        *id,
+				      const GValue       *value)
+{
+	if (!strcmp (id, "mode"))
+		glade_gnome_font_picker_set_mode (object, value);
+	else
+		GWA_GET_CLASS (GTK_TYPE_BUTTON)->set_property (adaptor,
+							       object,
+							       id, value);
+}
+
 GList * GLADEGNOME_API
 glade_gnome_font_picker_get_children (GladeWidgetAdaptor  *adaptor,
 				      GObject             *object)
 {
 	GtkWidget *child;
-	
-	g_return_val_if_fail (GNOME_IS_FONT_PICKER (object), NULL);
-	
+		
 	if ((child = gnome_font_picker_uw_get_widget (GNOME_FONT_PICKER (object))))
 		return g_list_append (NULL, G_OBJECT (child));
 	else
@@ -1329,7 +1255,6 @@ glade_gnome_font_picker_add_child (GladeWidgetAdaptor  *adaptor,
 				   GtkWidget           *container, 
 				   GtkWidget           *child)
 {
-	g_return_if_fail (GNOME_IS_FONT_PICKER (container));
 	gnome_font_picker_uw_set_widget (GNOME_FONT_PICKER (container), child);	
 }
 
@@ -1338,7 +1263,6 @@ glade_gnome_font_picker_remove_child (GladeWidgetAdaptor  *adaptor,
 				      GtkWidget           *container, 
 				      GtkWidget           *child)
 {
-	g_return_if_fail (GNOME_IS_FONT_PICKER (container));
 	gnome_font_picker_uw_set_widget (GNOME_FONT_PICKER (container), glade_placeholder_new ());
 }
 
@@ -1348,11 +1272,19 @@ glade_gnome_font_picker_replace_child (GladeWidgetAdaptor  *adaptor,
 				       GtkWidget           *current,
 				       GtkWidget           *new)
 {
-	g_return_if_fail (GNOME_IS_FONT_PICKER (container));
 	gnome_font_picker_uw_set_widget (GNOME_FONT_PICKER (container), new);
 }
 
 /* GnomeIconList */
+void GLADEGNOME_API
+glade_gnome_icon_list_post_create (GladeWidgetAdaptor  *adaptor,
+				   GObject             *object, 
+				   GladeCreateReason    reason)
+{
+	/* Freeze the widget so we dont get the signals that cause a segfault */
+	gnome_icon_list_freeze (GNOME_ICON_LIST (object));
+}
+
 static GType
 glade_gnome_icon_list_selection_mode_get_type (void)
 {
@@ -1379,48 +1311,30 @@ glade_gnome_icon_list_selection_mode_spec (void)
 }
 
 void GLADEGNOME_API
-glade_gnome_icon_list_set_selection_mode (GObject *object, GValue *value)
+glade_gnome_icon_list_set_property (GladeWidgetAdaptor *adaptor,
+				    GObject            *object,
+				    const gchar        *id,
+				    const GValue       *value)
 {
-	g_return_if_fail (GNOME_IS_ICON_LIST (object));
-	
-	gnome_icon_list_set_selection_mode (GNOME_ICON_LIST (object),
-					    g_value_get_enum (value));
-}
-
-void GLADEGNOME_API
-glade_gnome_icon_list_set_icon_width (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_ICON_LIST (object));
-	
-	gnome_icon_list_set_icon_width (GNOME_ICON_LIST (object),
-					g_value_get_int (value));
-}
-
-void GLADEGNOME_API
-glade_gnome_icon_list_set_row_spacing (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_ICON_LIST (object));
-	
-	gnome_icon_list_set_row_spacing (GNOME_ICON_LIST (object),
-					 g_value_get_int (value));
-}
-
-void GLADEGNOME_API
-glade_gnome_icon_list_set_column_spacing (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_ICON_LIST (object));
-	
-	gnome_icon_list_set_col_spacing (GNOME_ICON_LIST (object),
-					 g_value_get_int (value));
-}
-
-void GLADEGNOME_API
-glade_gnome_icon_list_set_text_spacing (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_ICON_LIST (object));
-	
-	gnome_icon_list_set_text_spacing (GNOME_ICON_LIST (object),
-					  g_value_get_int (value));
+	if (!strcmp (id, "selection-mode"))
+		gnome_icon_list_set_selection_mode (GNOME_ICON_LIST (object),
+						    g_value_get_enum (value));
+	else if (!strcmp (id, "icon-width"))
+		gnome_icon_list_set_icon_width (GNOME_ICON_LIST (object),
+						g_value_get_int (value));
+	else if (!strcmp (id, "row-spacing"))
+		gnome_icon_list_set_row_spacing (GNOME_ICON_LIST (object),
+						 g_value_get_int (value));	
+	else if (!strcmp (id, "column-spacing"))
+		gnome_icon_list_set_col_spacing (GNOME_ICON_LIST (object),
+						 g_value_get_int (value));
+	else if (!strcmp (id, "text-spacing"))
+		gnome_icon_list_set_text_spacing (GNOME_ICON_LIST (object),
+						  g_value_get_int (value));
+	else
+		GWA_GET_CLASS (GNOME_TYPE_CANVAS)->set_property (adaptor,
+								 object,
+								 id, value);
 }
 
 /* GnomePixmap */
@@ -1451,23 +1365,11 @@ glade_gnome_pixmap_set_filename_common (GObject *object)
 	return -1;
 }
 
-void GLADEGNOME_API
-glade_gnome_pixmap_set_filename (GObject *object, GValue *value)
-{
-	g_return_if_fail (GNOME_IS_PIXMAP (object));
-
-	if (glade_gnome_pixmap_set_filename_common (object))
-		gtk_image_set_from_pixbuf (GTK_IMAGE(object),
-					   GDK_PIXBUF (g_value_get_object (value)));
-}
-
 static void
 glade_gnome_pixmap_set_scaled_common (GObject *object,
-				      GValue *value,
+				      const GValue *value,
 				      const gchar *property)
 {
-	g_return_if_fail (GNOME_IS_PIXMAP (object));
-	
 	if (glade_gnome_pixmap_set_filename_common (object))
 	{
 		GladeWidget *gp = glade_widget_get_from_gobject (object);
@@ -1500,6 +1402,29 @@ glade_gnome_pixmap_set_scaled_height (GObject *object, GValue *value)
 	glade_gnome_pixmap_set_scaled_common (object, value, "scaled-width");
 }
 
+
+void GLADEGNOME_API
+glade_gnome_pixmap_set_property (GladeWidgetAdaptor *adaptor,
+				 GObject            *object,
+				 const gchar        *id,
+				 const GValue       *value)
+{
+	if (!strcmp (id, "filename"))
+	{
+		if (glade_gnome_pixmap_set_filename_common (object))
+			gtk_image_set_from_pixbuf (GTK_IMAGE(object),
+						   GDK_PIXBUF (g_value_get_object (value)));
+	}
+	else if (!strcmp (id, "scaled-width") ||
+		 !strcmp (id, "scaled-height"))
+		glade_gnome_pixmap_set_scaled_common (object, value, id);
+	else
+		GWA_GET_CLASS (GTK_TYPE_IMAGE)->set_property (adaptor,
+							      object,
+							      id, value);
+}
+
+
 /*
  BBBB     OOOO    NN   NN    OOOO    BBBB     OOOO
  BB  B   OO  OO   NNN  NN   OO  OO   BB  B   OO  OO
@@ -1598,32 +1523,22 @@ glade_gnome_bdb_get_band (GList *bands, GtkWidget *widget)
 static BonoboDockBand *
 glade_gnome_bd_get_band (BonoboDock *dock, GtkWidget *widget)
 {
-	BonoboDockBand *retval;
+	BonoboDockBand *retval = NULL;
 	
-	retval = glade_gnome_bdb_get_band (dock->top_bands, widget);
-	if (retval) return retval;
-
-	retval = glade_gnome_bdb_get_band (dock->bottom_bands, widget);
-	if (retval) return retval;
-
-	retval = glade_gnome_bdb_get_band (dock->right_bands, widget);
-	if (retval) return retval;
-
-	retval = glade_gnome_bdb_get_band (dock->left_bands, widget);
-	if (retval) return retval;
-		
-	return NULL;
+	if ((retval = glade_gnome_bdb_get_band (dock->top_bands, widget))    ||
+	    (retval = glade_gnome_bdb_get_band (dock->bottom_bands, widget)) ||
+	    (retval = glade_gnome_bdb_get_band (dock->right_bands, widget))  ||
+	    (retval = glade_gnome_bdb_get_band (dock->left_bands, widget)));
+	
+	return retval;
 }
 
 /* BonoboDock */
 void GLADEGNOME_API
 glade_gnome_bonobodock_add_child (GladeWidgetAdaptor  *adaptor,
-				 GObject             *object,
-				 GObject             *child)
+				  GObject             *object,
+				  GObject             *child)
 {
-	g_return_if_fail (BONOBO_IS_DOCK (object));
-	
-
 	if (BONOBO_IS_DOCK_ITEM (child))
 		bonobo_dock_add_item (BONOBO_DOCK (object), BONOBO_DOCK_ITEM (child),
 				      0,0,0,0, TRUE);
@@ -1637,9 +1552,7 @@ glade_gnome_bonobodock_remove_child (GladeWidgetAdaptor  *adaptor,
 				     GObject             *child)
 {
 	BonoboDockBand *band;
-	
-	g_return_if_fail (BONOBO_IS_DOCK (object));
-	
+		
 	band = glade_gnome_bd_get_band (BONOBO_DOCK (object), GTK_WIDGET (child));
 	
 	gtk_container_remove (GTK_CONTAINER (band), GTK_WIDGET (child));
@@ -1652,9 +1565,7 @@ glade_gnome_bonobodock_get_children (GladeWidgetAdaptor  *adaptor,
 	GList            *list = NULL, *l;
 	GtkWidget        *client_area;
 	BonoboDockLayout *layout;
-	
-	g_return_val_if_fail (BONOBO_IS_DOCK (object), NULL);
-	
+		
 	layout      = bonobo_dock_get_layout (BONOBO_DOCK (object));
 	client_area = bonobo_dock_get_client_area (BONOBO_DOCK (object));
 	
@@ -1676,7 +1587,6 @@ glade_gnome_bonobodock_replace_child (GladeWidgetAdaptor  *adaptor,
 				      GtkWidget           *current,
 				      GtkWidget           *new)
 {
-	g_return_if_fail (BONOBO_IS_DOCK (container));
 	bonobo_dock_set_client_area (BONOBO_DOCK (container), new);
 }
 
@@ -1723,8 +1633,9 @@ glade_gnome_bonobodock_set_child_property (GladeWidgetAdaptor  *adaptor,
 	GtkWidget *wchild;
 	gboolean new_band = FALSE;
 	
-	g_return_if_fail (BONOBO_IS_DOCK (container));
-	g_return_if_fail (BONOBO_IS_DOCK_ITEM (child));
+	if (!BONOBO_IS_DOCK_ITEM (child))
+		/* Ignore packing properties of client area */
+		return;
 
 	dock = BONOBO_DOCK (container);
 	item = BONOBO_DOCK_ITEM (child);
@@ -1782,8 +1693,14 @@ glade_gnome_bonobodock_get_child_property (GladeWidgetAdaptor  *adaptor,
 {
 	BonoboDockPlacement placement;
 	guint band_num, band_position, offset;
-	
-	g_return_if_fail (BONOBO_IS_DOCK (container));
+
+	if (!BONOBO_IS_DOCK_ITEM (child))
+		/* Ignore packing properties of client area, 
+		 *
+		 * FIXME: packing properties should actually be removed 
+		 * from client area children using glade_widget_remove_property();
+		 */
+		return;
 
 	if (strcmp ("behavior", property_name) == 0)
 	{
@@ -1807,10 +1724,16 @@ glade_gnome_bonobodock_get_child_property (GladeWidgetAdaptor  *adaptor,
 }
 
 void GLADEGNOME_API
-glade_gnome_bonobodock_set_allow_floating (GObject *object, GValue *value)
+glade_gnome_bonobodock_set_property (GladeWidgetAdaptor *adaptor,
+				     GObject            *object,
+				     const gchar        *id,
+				     const GValue       *value)
 {
-	g_return_if_fail (BONOBO_IS_DOCK (object));
-	bonobo_dock_allow_floating_items (BONOBO_DOCK (object),
-					  g_value_get_boolean (value));
+	if (!strcmp (id, "allow-floating"))
+		bonobo_dock_allow_floating_items (BONOBO_DOCK (object),
+						  g_value_get_boolean (value));
+	else
+		GWA_GET_CLASS (GTK_TYPE_CONTAINER)->set_property (adaptor,
+								  object,
+								  id, value);
 }
-
