@@ -33,6 +33,7 @@
 #include "glade.h"
 #include "glade-design-view.h"
 #include "glade-project-window.h"
+#include "glade-binding.h"
 
 #define GLADE_ACTION_GROUP_STATIC "GladeStatic"
 #define GLADE_ACTION_GROUP_PROJECT "GladeProject"
@@ -66,7 +67,16 @@ struct _GladeProjectWindowPrivate
 	GtkActionGroup *projects_list_menu_actions;/* Projects list menu actions */
 	
 	GtkLabel *label; /* the title of property editor dock */
+	
+	GtkRecentManager *recent_manager;
+	GtkWidget *recent_menu;
 
+	GtkWidget *console;
+
+	gchar *default_path; /* the default path for open/save operations */
+	
+	GtkToggleToolButton *selector_button; /* the widget selector button (replaces the one in the palette) */
+	
 };
 
 const gint   GLADE_WIDGET_TREE_WIDTH      = 230;
@@ -77,6 +87,7 @@ static gpointer parent_class = NULL;
 
 static void gpw_refresh_undo_redo (GladeProjectWindow *gpw);
 
+static void gpw_recent_chooser_item_activated_cb (GtkRecentChooser *chooser, GladeProjectWindow *gpw);
 
 G_DEFINE_TYPE(GladeProjectWindow, glade_project_window, GLADE_TYPE_APP)
 
@@ -109,6 +120,92 @@ gpw_refresh_title (GladeProjectWindow *gpw)
 	g_free (title);
 }
 
+static const gchar*
+gpw_get_default_path (GladeProjectWindow *gpw)
+{
+	return gpw->priv->default_path;
+}
+
+static void
+update_default_path (GladeProjectWindow *gpw, GladeProject *project)
+{
+	gchar *path;
+	
+	g_return_if_fail (project->path != NULL);
+
+	path = g_path_get_dirname (project->path);
+
+	g_free (gpw->priv->default_path);
+	gpw->priv->default_path = g_strdup (path);
+
+	g_free (path);
+}
+
+static gboolean
+gpw_window_state_event_cb (GtkWidget *widget,
+			   GdkEventWindowState *event,
+			   GladeProjectWindow *gpw)
+{
+	if (event->changed_mask &
+	    (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN))
+	{
+		gboolean show;
+
+		show = !(event->new_window_state &
+			(GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN));
+
+		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (gpw->priv->statusbar), show);
+	}
+
+	return FALSE;
+}
+
+static GtkWidget *
+create_recent_chooser_menu (GladeProjectWindow *gpw, GtkRecentManager *manager)
+{
+	GtkWidget *recent_menu;
+	GtkRecentFilter *filter;
+
+	recent_menu = gtk_recent_chooser_menu_new_for_manager (manager);
+
+	gtk_recent_chooser_set_local_only (GTK_RECENT_CHOOSER (recent_menu), TRUE);
+	gtk_recent_chooser_set_show_icons (GTK_RECENT_CHOOSER (recent_menu), FALSE);
+	gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (recent_menu), GTK_RECENT_SORT_MRU);
+	gtk_recent_chooser_menu_set_show_numbers (GTK_RECENT_CHOOSER_MENU (recent_menu), TRUE);
+
+	filter = gtk_recent_filter_new ();
+	gtk_recent_filter_add_application (filter, g_get_application_name());
+	gtk_recent_chooser_set_filter (GTK_RECENT_CHOOSER (recent_menu), filter);
+
+	return recent_menu;
+}
+
+static void
+gpw_window_screen_changed_cb (GtkWidget *widget,
+			      GdkScreen *old_screen,
+			      GladeProjectWindow *gpw)
+{
+	GtkWidget *menu_item;
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);	
+
+	gpw->priv->recent_manager = gtk_recent_manager_get_for_screen (screen);
+
+	gtk_menu_detach (GTK_MENU (gpw->priv->recent_menu));
+	g_object_unref (G_OBJECT (gpw->priv->recent_menu));
+	
+	gpw->priv->recent_menu = create_recent_chooser_menu (gpw, gpw->priv->recent_manager);
+
+	g_signal_connect (gpw->priv->recent_menu,
+			  "item-activated",
+			  G_CALLBACK (gpw_recent_chooser_item_activated_cb),
+			  gpw);
+			  
+	menu_item = gtk_ui_manager_get_widget (gpw->priv->ui, "/MenuBar/FileMenu/OpenRecent");
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), gpw->priv->recent_menu);
+}
+
 static void
 project_selection_changed_cb (GladeProject *project, GladeProjectWindow *gpw)
 {
@@ -302,6 +399,24 @@ gpw_construct_dock_item (GladeProjectWindow *gpw, const gchar *title, GtkWidget
 	return vbox;
 }
 
+void
+on_palette_toggled (GladePalette *palette, GladeProjectWindow *gpw)
+{
+	if (glade_palette_get_current_item (palette))
+		gtk_toggle_tool_button_set_active (gpw->priv->selector_button, FALSE);	
+	else
+		gtk_toggle_tool_button_set_active (gpw->priv->selector_button, TRUE);			
+}
+
+void
+on_selector_button_toggled (GtkToggleToolButton *button, GladeProjectWindow *gpw)
+{
+	if (gtk_toggle_tool_button_get_active (gpw->priv->selector_button))
+		glade_palette_deselect_current_item (glade_app_get_palette(), FALSE);
+	else if (glade_palette_get_current_item (glade_app_get_palette()) == FALSE)
+		gtk_toggle_tool_button_set_active (gpw->priv->selector_button, TRUE);
+}
+
 static void
 gpw_set_sensitivity_according_to_project (GladeProjectWindow *window, GladeProject *project)
 {
@@ -446,16 +561,85 @@ gpw_refresh_projects_list_menu (GladeProjectWindow *window)
 	p->projects_list_menu_ui_id = id;
 }
 
+
+void
+gpw_recent_add (GladeProjectWindow *gpw, const gchar *path)
+{
+	GtkRecentData *recent_data;
+	gchar *uri;
+	GError *error = NULL;
+
+	uri = g_filename_to_uri (path, NULL, &error);
+	if (error)
+	{	
+		g_warning ("Could not convert uri \"%s\" to a local path: %s", uri, error->message);
+		g_error_free (error);
+		return;
+	}
+
+	recent_data = g_slice_new (GtkRecentData);
+
+	recent_data->display_name   = NULL;
+	recent_data->description    = NULL;
+	recent_data->mime_type      = "application/x-glade";
+	recent_data->app_name       = (gchar *) g_get_application_name ();
+	recent_data->app_exec       = g_strjoin (" ", g_get_prgname (), "%u", NULL);
+	recent_data->groups         = NULL;
+	recent_data->is_private     = FALSE;
+
+	if (!gtk_recent_manager_add_full (gpw->priv->recent_manager,
+				          uri,
+				          recent_data))
+	{
+      		g_warning ("Unable to add '%s' to the list of recently used documents", uri);
+	}
+
+	g_free (uri);
+	g_free (recent_data->app_exec);
+	g_slice_free (GtkRecentData, recent_data);
+
+}
+
+void
+gpw_recent_remove (GladeProjectWindow *gpw, const gchar *path)
+{
+	gchar *uri;
+	GError *error = NULL;
+
+	uri = g_filename_to_uri (path, NULL, &error);
+	if (error)
+	{	
+		g_warning ("Could not convert uri \"%s\" to a local path: %s", uri, error->message);
+		g_error_free (error);
+		return;
+	}
+	
+	gtk_recent_manager_remove_item (gpw->priv->recent_manager, uri, &error);
+	if (error)
+	{
+		g_warning ("Could not remove recent-files uri \"%s\": %s", uri, error->message);	
+	}
+	
+	g_free (uri);
+}
+
 static void
 gpw_open_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
-	GtkWidget           *filechooser;
-	gchar               *path = NULL;
+	GtkWidget *filechooser;
+	gchar     *path = NULL, *default_path;
 
 	filechooser = glade_util_file_dialog_new (_("Open\342\200\246"), GTK_WINDOW (gpw->priv->window),
 						   GLADE_FILE_DIALOG_ACTION_OPEN);
 
 
+	default_path = g_strdup (gpw_get_default_path (gpw));
+	if (default_path != NULL)
+	{
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filechooser), default_path);
+		g_free (default_path);
+	}
+
 	if (gtk_dialog_run (GTK_DIALOG(filechooser)) == GTK_RESPONSE_OK)
 		path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
 
@@ -494,6 +678,9 @@ gpw_save (GladeProjectWindow *gpw, GladeProject *project, const gchar *path)
 	/* Get display_name here, it could have changed with "Save As..." */
 	display_name = glade_project_display_name (project, FALSE, FALSE, FALSE);
 
+	gpw_recent_add (gpw, project->path);
+	update_default_path (gpw, project);
+
 	/* refresh names */
 	gpw_refresh_title (gpw);
 	gpw_refresh_projects_list_item (gpw, project);
@@ -528,9 +715,20 @@ gpw_save_as (GladeProjectWindow *gpw)
 						  GLADE_FILE_DIALOG_ACTION_SAVE);
 
 	if (project->path)
+	{
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (filechooser), project->path);
+	}
 	else	
+	{
+		gchar *default_path = g_strdup (gpw_get_default_path (gpw));
+		if (default_path != NULL)
+		{
+			gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filechooser), default_path);
+			g_free (default_path);
+		}
+
 		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (filechooser), project->name);
+	}
 	
  	if (gtk_dialog_run (GTK_DIALOG(filechooser)) == GTK_RESPONSE_OK)
 		path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
@@ -663,11 +861,19 @@ gpw_confirm_close_project (GladeProjectWindow *gpw, GladeProject *project)
 		{
 			GtkWidget *filechooser;
 			gchar *path = NULL;
+			gchar *default_path;
 
 			filechooser =
 				glade_util_file_dialog_new (_("Save\342\200\246"),
 							    GTK_WINDOW (gpw->priv->window),
 							    GLADE_FILE_DIALOG_ACTION_SAVE);
+	
+			default_path = g_strdup (gpw_get_default_path (gpw));
+			if (default_path != NULL)
+			{
+				gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filechooser), default_path);
+				g_free (default_path);
+			}
 
 			gtk_file_chooser_set_current_name
 				(GTK_FILE_CHOOSER (filechooser), project->name);
@@ -744,7 +950,7 @@ gpw_close_cb (GtkAction *action, GladeProjectWindow *gpw)
 static void
 gpw_quit_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
-/*	GList *list;
+	GList *list;
 
 	for (list = glade_app_get_projects (); list; list = list->next)
 	{
@@ -761,10 +967,11 @@ gpw_quit_cb (GtkAction *action, GladeProjectWindow *gpw)
 	while (glade_app_get_projects ())
 	{
 		GladeProject *project = GLADE_PROJECT (glade_app_get_projects ()->data);
-		do_close (gpw, project);
+		do_close (gpw, glade_design_view_get_from_project (project));
 	}
-*/
+
 	gtk_main_quit ();
+
 }
 
 static void
@@ -818,6 +1025,55 @@ gpw_redo_cb (GtkAction *action, GladeProjectWindow *gpw)
 	glade_app_command_redo ();
 }
 
+static gboolean
+gpw_hide_window_on_delete (GtkWidget *window, gpointer not_used, GtkUIManager *ui)
+{
+	glade_util_hide_window (GTK_WINDOW (window));
+	return TRUE;
+}
+
+static void
+gpw_construct_console (GladeProjectWindow *gpw)
+{
+	GtkWidget *window, *console, *notebook;
+	GList *list, *l;
+	
+	g_return_if_fail (gpw != NULL);
+
+	window = gpw->priv->console = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+
+	gtk_window_set_title (GTK_WINDOW (window), _("Glade's Script Console"));
+	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
+	gtk_widget_set_size_request (window, 640, 480);
+	
+	notebook = gtk_notebook_new ();
+	gtk_container_add (GTK_CONTAINER (window), notebook);
+	gtk_widget_show (notebook);
+	
+	for (list = l = glade_binding_get_all(); l && l->data; l = g_list_next (l))
+	{
+		GladeBinding *binding = l->data;
+		
+		if ((console = glade_binding_console_new (binding)))
+		{
+			gtk_notebook_append_page (GTK_NOTEBOOK (notebook), console,
+						  gtk_label_new (glade_binding_get_name (binding)));
+			gtk_widget_show (console);
+		}
+	}
+
+	if (list == NULL)
+		gtk_widget_set_sensitive (gtk_ui_manager_get_widget 
+					  (gpw->priv->ui, "/MenuBar/ViewMenu/Console"),
+					  FALSE);
+
+	g_list_free (list);
+	
+	/* Delete event, don't destroy it */
+	g_signal_connect (window, "delete-event",
+			  G_CALLBACK (gpw_hide_window_on_delete), gpw->priv->ui);
+}
+
 static void
 gpw_doc_search_cb (GladeEditor        *editor,
 		   const gchar        *book,
@@ -828,43 +1084,6 @@ gpw_doc_search_cb (GladeEditor        *editor,
 	glade_util_search_devhelp (book, page, search);
 }
 
-static gboolean
-gpw_hide_clipboard_view_on_delete (GtkWidget *clipboard_view, gpointer not_used,
-				   GtkUIManager *ui)
-{
-	glade_util_hide_window (GTK_WINDOW (clipboard_view));
-	return TRUE;
-}
-
-static void 
-gpw_create_clipboard_view (GladeProjectWindow *gpw)
-{
-	GtkWidget *view;
-	
-	view = glade_app_get_clipboard_view ();
-
-	g_signal_connect (G_OBJECT (view), "delete_event",
-			  G_CALLBACK (gpw_hide_clipboard_view_on_delete),
-			  gpw->priv->ui);
-}
-
-static void
-gpw_show_clipboard_view (GladeProjectWindow *gpw)
-{
-	static gboolean created = FALSE;
-	
-	g_return_if_fail (gpw != NULL);
-
-	if (!created)
-	{
-		gpw_create_clipboard_view (gpw);
-		created = TRUE;
-	}
-	
-	gtk_widget_show_all (GTK_WIDGET (glade_app_get_clipboard_view ()));
-        gtk_window_present (GTK_WINDOW (glade_app_get_clipboard_view ()));
-}
-
 static void
 glade_project_window_previous_project_cb (GtkAction *action, GladeProjectWindow *window)
 {
@@ -921,6 +1140,16 @@ gpw_notebook_switch_page_cb (GtkNotebook *notebook,
 
 }
 
+static void
+gpw_show_console_cb (GtkAction *action, GladeProjectWindow *gpw)
+{
+	g_return_if_fail (gpw != NULL);
+
+	gtk_widget_show (gpw->priv->console);
+
+	gtk_window_present (GTK_WINDOW (gpw->priv->console));
+}
+
 static void
 gpw_notebook_tab_added_cb (GtkNotebook *notebook,
 			   GladeDesignView *view,
@@ -992,6 +1221,31 @@ gpw_notebook_tab_removed_cb (GtkNotebook *notebook,
 
 }
 
+static void
+gpw_recent_chooser_item_activated_cb (GtkRecentChooser *chooser, GladeProjectWindow *gpw)
+{
+	gchar *uri, *path;
+	GError *error = NULL;
+
+	uri = gtk_recent_chooser_get_current_uri (chooser);
+
+	path = g_filename_from_uri (uri, NULL, NULL);
+	if (error)
+	{
+		g_warning ("Could not convert uri \"%s\" to a local path: %s", uri, error->message);
+		g_error_free (error);
+		return;
+	}
+	
+	if (!glade_project_window_open_project (gpw, path))
+	{
+		gpw_recent_remove (gpw, path);
+	}
+
+	g_free (uri);
+	g_free (path);
+}
+
 static void
 gpw_palette_appearance_change_cb (GtkRadioAction *action,
 				  GtkRadioAction *current,
@@ -1015,7 +1269,21 @@ gpw_palette_toggle_small_icons_cb (GtkAction *action, GladeProjectWindow *gpw)
 static void
 gpw_show_clipboard_cb (GtkAction *action, GladeProjectWindow *gpw)
 {
-	gpw_show_clipboard_view (gpw);
+	static GtkWidget *view = NULL;
+	
+	g_return_if_fail (gpw != NULL);
+
+	if (view == NULL)
+	{
+		view = glade_app_get_clipboard_view ();
+
+		g_signal_connect (view, "delete_event",
+				  G_CALLBACK (gpw_hide_window_on_delete),
+				  gpw->priv->ui);
+		gtk_widget_show_all (view);
+	}
+	
+        gtk_window_present (GTK_WINDOW (view));
 }
 
 static void
@@ -1121,6 +1389,7 @@ static const gchar *ui_info =
 "    <menu action='FileMenu'>\n"
 "      <menuitem action='New'/>\n"
 "      <menuitem action='Open'/>\n"
+"      <menuitem action='OpenRecent'/>\n"
 "      <separator/>\n"
 "      <menuitem action='Save'/>\n"
 "      <menuitem action='SaveAs'/>\n"
@@ -1139,6 +1408,7 @@ static const gchar *ui_info =
 "    </menu>\n"
 "    <menu action='ViewMenu'>\n"
 "      <menuitem action='Clipboard'/>\n"
+"      <menuitem action='Console'/>\n"
 "      <separator/>\n"
 "      <menu action='PaletteAppearance'>\n"
 "        <menuitem action='IconsAndLabels'/>\n"
@@ -1188,6 +1458,8 @@ static GtkActionEntry static_entries[] = {
 	
 	{ "Open", GTK_STOCK_OPEN, N_("_Open"),"<control>O",
 	  N_("Open a project"), G_CALLBACK (gpw_open_cb) },
+
+	{ "OpenRecent", NULL, N_("Open _Recent") },	
 	
 	{ "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 	  N_("Quit the program"), G_CALLBACK (gpw_quit_cb) },
@@ -1240,7 +1512,11 @@ static GtkActionEntry project_entries[] = {
 	{ "Clipboard", NULL, N_("_Clipboard"), NULL,
 	  N_("Show the clipboard"),
 	  G_CALLBACK (gpw_show_clipboard_cb) },
-
+	  
+	{ "Console", NULL, N_("C_onsole"), NULL,
+	  N_("Show Script-do console"),
+	  G_CALLBACK (gpw_show_console_cb) },
+	  
 	/* ProjectsMenu */
 	{ "PreviousProject", NULL, N_("_Previous Project"), "<control>Page_Up",
 	  N_("Activate previous project"), G_CALLBACK (glade_project_window_previous_project_cb) },
@@ -1440,7 +1716,9 @@ gpw_drag_data_received (GtkWidget *widget,
 		gchar *path = g_filename_from_uri (*str, NULL, &error);
 
 		if (path)
+		{
 			glade_project_window_open_project (window, path);
+		}
 		else
 		{
 			g_warning ("Could not convert uri to local path: %s", error->message); 
@@ -1461,6 +1739,33 @@ gpw_delete_event (GtkWindow *w, GdkEvent *event, GladeProjectWindow *gpw)
 	return TRUE;	
 }
 
+static GtkWidget *
+create_selector_tool_button (GtkToolbar *toolbar)
+{
+	GtkToolItem  *button;
+	GtkWidget    *image;
+	gchar        *image_path;
+	
+	image_path = g_build_filename (glade_pixmaps_dir, "selector.png", NULL);
+	image = gtk_image_new_from_file (image_path);
+	g_free (image_path);
+	
+	button = gtk_toggle_tool_button_new ();
+	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (button), TRUE);
+	
+	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (button), image);
+	
+	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (button),
+				   toolbar->tooltips,
+				   _("Select a widget"),
+				   NULL);
+	
+	gtk_widget_show (GTK_WIDGET (button));
+	gtk_widget_show (image);
+	
+	return GTK_WIDGET (button);
+}
+
 static void
 glade_project_window_create (GladeProjectWindow *gpw)
 {
@@ -1477,10 +1782,13 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	GtkWidget *palette;
 	GtkWidget *editor;
 	GtkWidget *dockitem;
+	GtkWidget *widget;
+	GtkToolItem *sep;
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gpw->priv->window = window;
 	gtk_window_maximize (GTK_WINDOW (window));
+	gtk_window_set_default_size (GTK_WINDOW (window), 720, 540);
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (window), vbox);
@@ -1500,20 +1808,24 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	hpaned1 = gtk_hpaned_new ();
 	hpaned2 = gtk_hpaned_new ();
 	vpaned = gtk_vpaned_new ();
-	
-	/* minimum size for far-right pane */
-	gtk_widget_set_size_request (vpaned, 310, -1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), hpaned1, TRUE, TRUE, 0);
 	gtk_paned_pack1 (GTK_PANED (hpaned1), hpaned2, TRUE, FALSE);
 	gtk_paned_pack2 (GTK_PANED (hpaned1), vpaned, FALSE, FALSE);
 
+	/* divider position between design area and editor/tree */
+	gtk_paned_set_position (GTK_PANED (hpaned1), 350);
+	/* divider position between tree and editor */	
+	gtk_paned_set_position (GTK_PANED (vpaned), 150);
+
+
 	gtk_widget_show_all (hpaned1);
 	gtk_widget_show_all (hpaned2);
 	gtk_widget_show_all (vpaned);
 
 	/* palette */
 	palette = GTK_WIDGET (glade_app_get_palette ());
+	glade_palette_set_show_selector_button (GLADE_PALETTE (palette), FALSE);
 	dockitem = gpw_construct_dock_item (gpw, _("Palette"), palette);
 	gtk_paned_pack1 (GTK_PANED (hpaned2), dockitem, FALSE, FALSE);
 
@@ -1524,7 +1836,7 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	gtk_paned_pack2 (GTK_PANED (hpaned2), gpw->priv->notebook, TRUE, FALSE);
 	gtk_widget_show (gpw->priv->notebook);
 
-	/* project view */	
+	/* project view */
 	project_view = GLADE_PROJECT_VIEW (glade_project_view_new ());
 	glade_app_add_project_view (project_view);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (project_view),
@@ -1540,7 +1852,7 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	gpw->priv->label = GTK_LABEL (g_object_get_data (G_OBJECT (dockitem), "dock-label"));
 	gtk_label_set_ellipsize	(GTK_LABEL (gpw->priv->label), PANGO_ELLIPSIZE_END);
 	gtk_misc_set_alignment (GTK_MISC (gpw->priv->label), 0, 0.5);
-	gtk_paned_pack2 (GTK_PANED (vpaned), dockitem, FALSE, FALSE);
+	gtk_paned_pack2 (GTK_PANED (vpaned), dockitem, TRUE, FALSE);
 
 	/* status bar */
 	statusbar = gpw_construct_statusbar (gpw);
@@ -1559,6 +1871,38 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	gtk_widget_set_sensitive (editor_item, FALSE);
 	gtk_widget_set_sensitive (docs_item, FALSE);
 	
+	/* Console */
+	gpw_construct_console (gpw);
+	
+	/* recent files */	
+	gpw->priv->recent_manager = gtk_recent_manager_get_for_screen (gtk_widget_get_screen (gpw->priv->window));
+
+	gpw->priv->recent_menu = create_recent_chooser_menu (gpw, gpw->priv->recent_manager);
+
+	g_signal_connect (gpw->priv->recent_menu,
+			  "item-activated",
+			  G_CALLBACK (gpw_recent_chooser_item_activated_cb),
+			  gpw);
+			  
+	widget = gtk_ui_manager_get_widget (gpw->priv->ui, "/MenuBar/FileMenu/OpenRecent");
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (widget), gpw->priv->recent_menu);		  
+	
+	/* palette selector button */
+	
+	gpw->priv->selector_button = GTK_TOGGLE_TOOL_BUTTON (create_selector_tool_button (GTK_TOOLBAR (toolbar)));	
+	
+	sep = gtk_separator_tool_item_new();
+	gtk_widget_show (GTK_WIDGET (sep));
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (sep), -1);
+	
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (gpw->priv->selector_button), -1);
+	
+	g_signal_connect (G_OBJECT (glade_app_get_palette ()), "toggled",
+			  G_CALLBACK (on_palette_toggled), gpw);
+	g_signal_connect (G_OBJECT (gpw->priv->selector_button), "toggled",
+			  G_CALLBACK (on_selector_button_toggled), gpw);
+	
+	
 	/* support for opening a file by dragging onto the project window */
 	gtk_drag_dest_set (GTK_WIDGET (window),
 			   GTK_DEST_DEFAULT_ALL,
@@ -1571,7 +1915,7 @@ glade_project_window_create (GladeProjectWindow *gpw)
 	g_signal_connect (G_OBJECT (window), "delete_event",
 			  G_CALLBACK (gpw_delete_event), gpw);
 			  
-	/* Connect signals notebook */
+	/* connect signals notebook */
 	g_signal_connect (gpw->priv->notebook,
 			  "switch-page",
 			  G_CALLBACK (gpw_notebook_switch_page_cb),
@@ -1584,6 +1928,16 @@ glade_project_window_create (GladeProjectWindow *gpw)
 			  "page-removed",
 			  G_CALLBACK (gpw_notebook_tab_removed_cb),
 			  gpw);
+			  
+			  
+	/* GtkWindow events */
+	g_signal_connect (gpw->priv->window, "screen-changed",
+			  G_CALLBACK (gpw_window_screen_changed_cb),
+			  gpw);
+			  
+	g_signal_connect (gpw->priv->window, "window-state-event",
+			  G_CALLBACK (gpw_window_state_event_cb),
+			  gpw);
 }
 
 static void
@@ -1619,12 +1973,12 @@ glade_project_window_new_project (GladeProjectWindow *gpw)
 	glade_project_window_add_project (gpw, project);
 }
 
-void
+gboolean
 glade_project_window_open_project (GladeProjectWindow *gpw, const gchar *path)
 {
 	GladeProject *project;
 
-	g_return_if_fail (path != NULL);
+	g_return_val_if_fail (path != NULL, FALSE);
 
 	/* dont allow more than one project with the same name to be
 	 * opened simultainiously.
@@ -1633,15 +1987,22 @@ glade_project_window_open_project (GladeProjectWindow *gpw, const gchar *path)
 	{
 		glade_util_ui_message (gpw->priv->window, GLADE_UI_WARN, 
 				       _("%s is already open"), path);
-		return;
+		return TRUE; /* let this pass */
 	}
 	
 	if ((project = glade_project_open (path)) == NULL)
-		return;
+		return FALSE;
 
 	glade_project_window_add_project (gpw, project);
+	
+	gpw_recent_add (gpw, project->path);
+	update_default_path (gpw, project);
+	
+	return TRUE;
 }
 
+
+
 static void
 glade_project_window_change_menu_label (GladeProjectWindow *gpw,
 					const gchar *path,
@@ -1728,6 +2089,8 @@ glade_project_window_show_all (GladeProjectWindow *gpw)
 static void
 glade_project_window_finalize (GObject *object)
 {
+	g_free (GLADE_PROJECT_WINDOW (object)->priv->default_path);
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1755,6 +2118,7 @@ glade_project_window_init (GladeProjectWindow *gpw)
 	gpw->priv = GLADE_PROJECT_WINDOW_GET_PRIVATE (gpw);
 	
 	gpw->priv->label = NULL;
+	gpw->priv->default_path = NULL;
 
 }
 
